@@ -1,5 +1,4 @@
-/* $Id: ttkPanedwindow.c,v 1.14.2.1 2010/08/26 02:06:09 hobbs Exp $
- *
+/*
  * Copyright (c) 2005, Joe English.  Freely redistributable.
  *
  * ttk::panedwindow widget implementation.
@@ -87,6 +86,7 @@ static Tk_OptionSpec PanedOptionSpecs[] = {
 	-1,Tk_Offset(Paned,paned.height),
 	0,0,GEOMETRY_CHANGED },
 
+    WIDGET_TAKEFOCUS_FALSE,
     WIDGET_INHERIT_OPTIONS(ttkCoreOptionSpecs)
 };
 
