@@ -7,9 +7,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkWinDialog.c,v 1.50.2.9 2010/05/19 11:22:59 nijtmans Exp $
- *
  */
 
 #define WINVER        0x0500   /* Requires Windows 2K definitions */
@@ -595,7 +592,8 @@ GetFileNameW(
     OFNData ofnData;
     int cdlgerr;
     int filterIndex = 0, result = TCL_ERROR, winCode, oldMode, i, multi = 0;
-    char *extension = NULL, *filter = NULL, *title = NULL;
+    int inValue, confirmOverwrite = 1;
+    char *extension = NULL, *title = NULL;
     Tk_Window tkwin = (Tk_Window) clientData;
     HWND hWnd;
     Tcl_Obj *filterObj = NULL, *initialTypeObj = NULL, *typeVariableObj = NULL;
@@ -606,17 +604,20 @@ GetFileNameW(
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     static CONST char *saveOptionStrings[] = {
 	"-defaultextension", "-filetypes", "-initialdir", "-initialfile",
-	"-parent", "-title", "-typevariable", NULL
+	"-parent", "-title", "-typevariable",
+	"-confirmoverwrite",
+	NULL
     };
     static CONST char *openOptionStrings[] = {
 	"-defaultextension", "-filetypes", "-initialdir", "-initialfile",
-	"-multiple", "-parent", "-title", "-typevariable", NULL
+	"-parent", "-title", "-typevariable",
+	"-multiple",
+	NULL
     };
-    CONST char **optionStrings;
-
     enum options {
-	FILE_DEFAULT,	FILE_TYPES,	FILE_INITDIR,	FILE_INITFILE,
-	FILE_MULTIPLE,	FILE_PARENT,	FILE_TITLE,     FILE_TYPEVARIABLE
+	FILE_DEFAULT,	FILE_TYPES,	FILE_INITDIR, FILE_INITFILE,
+	FILE_PARENT,	FILE_TITLE, FILE_TYPEVARIABLE,
+	FILE_MULTIPLE_OR_CONFIRMOW
     };
 
     file[0] = '\0';
@@ -624,16 +625,6 @@ GetFileNameW(
     Tcl_DStringInit(&utfFilterString);
     Tcl_DStringInit(&utfDirString);
 
-    /*
-     * Parse the arguments.
-     */
-
-    if (open) {
-	optionStrings = openOptionStrings;
-    } else {
-	optionStrings = saveOptionStrings;
-    }
-
     for (i = 1; i < objc; i += 2) {
 	int index;
 	char *string;
@@ -642,26 +633,12 @@ GetFileNameW(
 	optionPtr = objv[i];
 	valuePtr = objv[i + 1];
 
-	if (Tcl_GetIndexFromObj(interp, optionPtr, optionStrings,
+	if (Tcl_GetIndexFromObj(interp, optionPtr,
+		open ? openOptionStrings : saveOptionStrings,
 		"option", 0, &index) != TCL_OK) {
 	    goto end;
 	}
 
-	/*
-	 * We want to maximize code sharing between the open and save file
-	 * dialog implementations; in particular, the switch statement below.
-	 * We use different sets of option strings from the GetIndexFromObj
-	 * call above, but a single enumeration for both. The save file dialog
-	 * doesn't support -multiple, but it falls in the middle of the
-	 * enumeration. Ultimately, this means that when the index found by
-	 * GetIndexFromObj is >= FILE_MULTIPLE, when doing a save file dialog,
-	 * we have to increment the index, so that it matches the open file
-	 * dialog enumeration.
-	 */
-
-	if (!open && index >= FILE_MULTIPLE) {
-	    index++;
-	}
 	if (i + 1 == objc) {
 	    string = Tcl_GetString(optionPtr);
 	    Tcl_AppendResult(interp, "value for \"", string, "\" missing",
@@ -696,11 +673,6 @@ GetFileNameW(
 		    sizeof(file), NULL, NULL, NULL);
 	    Tcl_DStringFree(&ds);
 	    break;
-	case FILE_MULTIPLE:
-	    if (Tcl_GetBooleanFromObj(interp, valuePtr, &multi) != TCL_OK) {
-		return TCL_ERROR;
-	    }
-	    break;
 	case FILE_PARENT:
 	    tkwin = Tk_NameToWindow(interp, string, tkwin);
 	    if (tkwin == NULL) {
@@ -715,6 +687,16 @@ GetFileNameW(
 	    initialTypeObj = Tcl_ObjGetVar2(interp, typeVariableObj, NULL,
 		    TCL_GLOBAL_ONLY);
 	    break;
+	case FILE_MULTIPLE_OR_CONFIRMOW:
+	    if (Tcl_GetBooleanFromObj(interp, valuePtr, &inValue) != TCL_OK) {
+		return TCL_ERROR;
+	    }
+	    if (open) {
+		multi = inValue;
+	    } else {
+		confirmOverwrite = inValue;
+	    }
+	    break;
 	}
     }
 
@@ -722,7 +704,6 @@ GetFileNameW(
 	    &filterIndex) != TCL_OK) {
 	goto end;
     }
-    filter = Tcl_DStringValue(&utfFilterString);
 
     Tk_MakeWindowExist(tkwin);
     hWnd = Tk_GetHWND(Tk_WindowId(tkwin));
@@ -738,13 +719,13 @@ GetFileNameW(
     ofn.lpstrFile = (WCHAR *) file;
     ofn.nMaxFile = TK_MULTI_MAX_PATH;
     ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_NOCHANGEDIR
-	    | OFN_EXPLORER | OFN_ENABLEHOOK;
+	    | OFN_EXPLORER | OFN_ENABLEHOOK| OFN_ENABLESIZING;
     ofn.lpfnHook = (LPOFNHOOKPROC) OFNHookProcW;
     ofn.lCustData = (LPARAM) &ofnData;
 
     if (open != 0) {
 	ofn.Flags |= OFN_FILEMUSTEXIST;
-    } else {
+    } else if (confirmOverwrite) {
 	ofn.Flags |= OFN_OVERWRITEPROMPT;
     }
     if (tsdPtr->debugFlag != 0) {
@@ -1026,17 +1007,20 @@ OFNHookProcW(
 	    dirsize = SendMessageW(hdlg, CDM_GETFOLDERPATH, 0, 0);
 	    buffersize = (selsize + dirsize + 1) * 2;
 
-	    if (selsize > 1) {
+	    /*
+	     * Just empty the buffer if dirsize indicates an error [Bug 3071836]
+	     */
+	    if ((selsize > 1) && (dirsize > 0)) {
 		if (ofnData->dynFileBufferSize < buffersize) {
 		    buffer = (WCHAR *) ckrealloc((char *) buffer, buffersize);
 		    ofnData->dynFileBufferSize = buffersize;
 		    ofnData->dynFileBuffer = (char *) buffer;
 		}
 
-		SendMessageW(hdlg, CDM_GETFOLDERPATH, dirsize, (int) buffer);
+		SendMessageW(hdlg, CDM_GETFOLDERPATH, dirsize, PTR2INT(buffer));
 		buffer += dirsize;
 
-		SendMessageW(hdlg, CDM_GETSPEC, selsize, (int) buffer);
+		SendMessageW(hdlg, CDM_GETSPEC, selsize, PTR2INT(buffer));
 
 		/*
 		 * If there are multiple files, delete the quotes and change
@@ -1133,7 +1117,7 @@ GetFileNameA(
     OFNData ofnData;
     int cdlgerr;
     int filterIndex = 0, result = TCL_ERROR, winCode, oldMode, i, multi = 0;
-    char *extension = NULL, *filter = NULL, *title = NULL;
+    char *extension = NULL, *title = NULL;
     Tk_Window tkwin = (Tk_Window) clientData;
     HWND hWnd;
     Tcl_Obj *filterObj = NULL, *initialTypeObj = NULL, *typeVariableObj = NULL;
@@ -1258,7 +1242,6 @@ GetFileNameA(
 	    &filterIndex) != TCL_OK) {
 	goto end;
     }
-    filter = Tcl_DStringValue(&utfFilterString);
 
     Tk_MakeWindowExist(tkwin);
     hWnd = Tk_GetHWND(Tk_WindowId(tkwin));
@@ -1572,16 +1555,19 @@ OFNHookProcA(
 	    dirsize = SendMessage(hdlg, CDM_GETFOLDERPATH, 0, 0);
 	    buffersize = selsize + dirsize + 1;
 
-	    if (selsize > 1) {
+	    /*
+	     * Just empty the buffer if dirsize indicates an error [Bug 3071836]
+	     */
+	    if ((selsize > 1) && (dirsize > 0)) {
 		if (ofnData->dynFileBufferSize < buffersize) {
 		    buffer = ckrealloc(buffer, buffersize);
 		    ofnData->dynFileBufferSize = buffersize;
 		    ofnData->dynFileBuffer = buffer;
 		}
 
-		SendMessage(hdlg, CDM_GETFOLDERPATH, dirsize, (int) buffer);
+		SendMessage(hdlg, CDM_GETFOLDERPATH, dirsize, PTR2INT(buffer));
 		buffer += dirsize;
-		SendMessage(hdlg, CDM_GETSPEC, selsize, (int) buffer);
+		SendMessage(hdlg, CDM_GETSPEC, selsize, PTR2INT(buffer));
 
 		/*
 		 * If there are multiple files, delete the quotes and change
@@ -2365,6 +2351,9 @@ Tk_MessageBoxObjCmd(
 	}
     }
 
+    while (!Tk_IsTopLevel(parent)) {
+	parent = Tk_Parent(parent);
+    }
     Tk_MakeWindowExist(parent);
     hWnd = Tk_GetHWND(Tk_WindowId(parent));
 
@@ -2394,7 +2383,7 @@ Tk_MessageBoxObjCmd(
 	flags = buttonFlagMap[defaultBtnIdx];
     }
 
-    flags |= icon | type | MB_SYSTEMMODAL;
+    flags |= icon | type | MB_TASKMODAL | MB_SETFOREGROUND;
 
     tmpObj = messageObj ? Tcl_DuplicateObj(messageObj)
 	    : Tcl_NewUnicodeObj(NULL, 0);
